@@ -13,6 +13,7 @@
 #     name: python3
 # ---
 
+# +
 """autoui is used to automatically create ipywidget user input (UI) form from a pydantic schema.
 
 This module maps the pydantic fields to appropriate widgets based on type to display the data in the UI.
@@ -22,7 +23,7 @@ This information can also be stored to file.
 Example:
     see example for a pydantic schema that can be automatically converted into a 
     ipywidgets UI. Currently nesting is not supported::
-    
+
         from ipyautoui.constants import DISPLAY_AUTOUI_SCHEMA_EXAMPLE
         DISPLAY_AUTOUI_SCHEMA_EXAMPLE()
 """
@@ -44,6 +45,11 @@ from ipyautoui.custom.buttonbars import SaveButtonBar
 from ipyautoui.custom.showhide import ShowHide
 from ipyautoui.autowidgets import Nullable
 
+logger = logging.getLogger(__name__)
+
+
+# -
+
 
 def _init_widgets_and_labels(
     pr: ty.Dict,
@@ -137,21 +143,36 @@ def horizontal_row_nested(widget, label, auto_open=False):
     )
 
 
-def horizontal_row_simple(widget, label):
-    return w.HBox([widget, w.HTML(label)])
+def add_spacer_to_form_with_nullables(children, widget, contains_nullable):
+    if contains_nullable and not isinstance(widget, Nullable):
+        children = [w.HBox(layout={"width": "45px"})] + children
+    return children
 
 
-def vertical_row(widget, label, auto_open=None):
+def horizontal_row_simple(widget, label, contains_nullable=False):
+    children = [widget, w.HTML(label)]
+    children = add_spacer_to_form_with_nullables(children, widget, contains_nullable)
+    return w.HBox(children)
+
+
+def vertical_row(widget, label, auto_open=None, contains_nullable=False):
+    children = [w.HBox(layout={"width": "40px"}), widget]
+    children = add_spacer_to_form_with_nullables(children, widget, contains_nullable)
     return w.VBox(
         [
             w.HTML(label),
-            w.HBox([w.HBox(layout={"width": "40px"}), widget]),
+            w.HBox(children),
         ]
     )
 
 
 def create_row(
-    widget, label, align_horizontal=True, auto_open=False, nested_widgets=None
+    widget,
+    label,
+    align_horizontal=True,
+    auto_open=False,
+    nested_widgets=None,
+    contains_nullable=False,
 ):
     """creates a row for a given widget. applies
 
@@ -171,9 +192,11 @@ def create_row(
         if True in [isinstance(widget, w) for w in nested_widgets]:
             return horizontal_row_nested(widget, label, auto_open=auto_open)
         else:
-            return horizontal_row_simple(widget, label)
+            return horizontal_row_simple(
+                widget, label, contains_nullable=contains_nullable
+            )
     else:
-        return vertical_row(widget, label)
+        return vertical_row(widget, label, contains_nullable=contains_nullable)
 
 
 def show_hide_widget(widget, show: bool):
@@ -187,7 +210,15 @@ def show_hide_widget(widget, show: bool):
 
 
 # +
-class AutoObjectShowRaw(w.VBox):
+def make_bold(s: str) -> str:
+    return f"<big><b>{s}</b></big>"
+
+
+class AutoObjectFormLayout(w.VBox):
+
+    show_description = tr.Bool(default_value=True)
+    show_title = tr.Bool(default_value=True)
+    show_savebuttonbar = tr.Bool(default_value=False)
     fn_onshowraw = tr.Callable(default_value=lambda: "{'test':'json'}")
     fn_onhideraw = tr.Callable(default_value=lambda: None)
     show_raw = tr.Bool(default_value=False)
@@ -196,16 +227,52 @@ class AutoObjectShowRaw(w.VBox):
     def _observe_show_raw(self, change):
         show_hide_widget(self.bn_showraw, self.show_raw)
 
-    def __init__(self):
+    @tr.observe("show_description")
+    def _observe_show_description(self, change):
+        show_hide_widget(self.description, self.show_description)
+
+    @tr.observe("show_title")
+    def _observe_show_title(self, change):
+        show_hide_widget(self.title, self.show_title)
+
+    @tr.observe("show_savebuttonbar")
+    def _observe_show_savebuttonbar(self, change):
+        show_hide_widget(self.savebuttonbar, self.show_savebuttonbar)
+
+    def __init__(self, fns_onsave=None, fns_onrevert=None, **kwargs):
+
+        self._init_form()
+        self._init_bn_showraw_controls()
+        self.fn_onshowraw = self.display_showraw
+        self.fn_onhideraw = self.display_ui
+        if fns_onsave is not None:
+            self.fns_onsave = fns_onsave
+        if fns_onrevert is not None:
+            self.fns_onrevert = fns_onrevert
         super().__init__(
             layout=w.Layout(
                 width="100%",
                 display="flex",
                 flex="flex-grow",
-            )
+            ),
+            **kwargs,
         )
+        self.children = [
+            self.savebuttonbar,
+            self.hbx_title,
+            self.description,
+            self.autowidget,
+            self.vbx_showraw,
+        ]
+
+    def _init_form(self):
+        self.autowidget = w.VBox()
+        self.hbx_title = w.HBox()
+        self.savebuttonbar = SaveButtonBar(layout={"display": "None"})  #
+        self.title = w.HTML()
         self._init_bn_showraw()
-        self._init_bn_showraw_controls()
+        self.hbx_title.children = [self.bn_showraw, self.title]
+        self.description = w.HTML()  #
 
     def _init_bn_showraw(self):
         self.bn_showraw = w.ToggleButton(
@@ -238,68 +305,6 @@ class AutoObjectShowRaw(w.VBox):
             self.fn_onhideraw()
             self.vbx_showraw.layout.display = "None"
 
-
-if __name__ == "__main__":
-    ui_showraw = AutoObjectShowRaw()
-    display(ui_showraw.bn_showraw)
-    display(ui_showraw.vbx_showraw)
-# -
-
-if __name__ == "__main__":
-    ui_showraw.show_raw = True
-
-
-# +
-def make_bold(s: str) -> str:
-    return f"<big><b>{s}</b></big>"
-
-
-class AutoObjectFormLayout(AutoObjectShowRaw):
-
-    show_description = tr.Bool(default_value=True)
-    show_title = tr.Bool(default_value=True)
-    show_savebuttonbar = tr.Bool(default_value=False)
-
-    @tr.observe("show_description")
-    def _observe_show_description(self, change):
-        show_hide_widget(self.description, self.show_description)
-
-    @tr.observe("show_title")
-    def _observe_show_title(self, change):
-        show_hide_widget(self.title, self.show_title)
-
-    @tr.observe("show_savebuttonbar")
-    def _observe_show_savebuttonbar(self, change):
-        show_hide_widget(self.savebuttonbar, self.show_savebuttonbar)
-
-    def __init__(self, fns_onsave=None, fns_onrevert=None):
-        super().__init__()
-        self._init_form()
-        self._init_bn_showraw_controls()
-        self.fn_onshowraw = self.display_showraw
-        self.fn_onhideraw = self.display_ui
-        if fns_onsave is not None:
-            self.fns_onsave = fns_onsave
-        if fns_onrevert is not None:
-            self.fns_onrevert = fns_onrevert
-
-    def _init_form(self):
-        self.autowidget = w.VBox()
-        self.hbx_title = w.HBox()
-        self.savebuttonbar = SaveButtonBar()
-        self.savebuttonbar.layout.display = "None"
-        self.title = w.HTML()
-        self._init_bn_showraw()
-        self.hbx_title.children = [self.bn_showraw, self.title]
-        self.description = w.HTML()  #
-        self.children = [
-            self.savebuttonbar,
-            self.hbx_title,
-            self.description,
-            self.autowidget,
-            self.vbx_showraw,
-        ]
-
     def display_ui(self):
         self.autowidget.layout.display = ""
 
@@ -334,6 +339,11 @@ class AutoObjectFormLayout(AutoObjectShowRaw):
             raise ValueError("fns_onrevert must be a callable or list of callables")
 
 
+if __name__ == "__main__":
+    ui = AutoObjectFormLayout(description="description", show_savebuttonbar=True)
+    display(ui)
+
+# +
 def demo_autoobject_form(title="test", description="a description of the title"):
     """for docs and testing only..."""
     from ipyautoui.custom.buttonbars import SaveButtonBar
@@ -362,6 +372,9 @@ def demo_autoobject_form(title="test", description="a description of the title")
 if __name__ == "__main__":
     form = demo_autoobject_form()
     display(form)
+
+# +
+# form.show_raw = True
 # -
 
 if __name__ == "__main__":
@@ -383,16 +396,76 @@ def autogen(schema) -> object:
 
 # +
 class AutoObject(AutoObjectFormLayout):  # w.VBox
-    """creates an ipywidgets form from a json-schema or pydantic model. datatype must be "object" """
+    """creates an ipywidgets form from a json-schema or pydantic model.
+    datatype must be "object"
+
+    Attributes:
+        _value (dict): use `value` to set and get. the value of the form. this is a dict of the form {key: value}
+        fdir (path, optional): fdir to work from. useful for widgets that link to files. Defaults to None.
+        align_horizontal (bool, optional): aligns widgets horizontally. Defaults to True.
+        nested_widgets (list, optional): allows user to indicate widgets that should be show / hide type. Defaults to [].
+        auto_open (bool, optional): automatically opens the nested_widget. Defaults to True.
+        order (list): allows user to re-specify the order for widget rows to appear by key name in self.di_widgets
+        order_can_hide_rows (bool): allows user to hide rows by removing them from the order list.
+        insert_rows (dict): e.g. {3:w.Button()}. allows user to insert a widget into the rows. its presence
+            is ignored by the widget otherwise.
+        disabled (bool, optional): disables all widgets. If widgets are disabled
+            using schema kwargs this is remembered when re-enabled. Defaults to False.
+
+        # inherited from AutoObjectFormLayout
+
+        show_raw (bool, optional): show the raw json. Defaults to False.
+        show_description (bool, optional): show the description. Defaults to True.
+        show_title (bool, optional): show the title. Defaults to True.
+        show_savebuttonbar (bool, optional): show the savebuttonbar. Defaults to True.
+
+    """
 
     _value = tr.Dict(allow_none=True)
-    fdir = tr.Unicode(allow_none=True)
+    fdir = tr.Unicode(default_value=None, allow_none=True)
     align_horizontal = tr.Bool(default_value=True)
     auto_open = tr.Bool(default_value=True)
     nested_widgets = tr.List()
     order = tr.List(default_value=None, allow_none=True)
     order_can_hide_rows = tr.Bool(default_value=True)
     insert_rows = tr.Dict(default_value=None, allow_none=True)
+    disabled = tr.Bool(default_value=False)
+
+    def __init__(
+        self,
+        schema,
+        by_alias=False,
+        value=None,
+        update_map_widgets=None,
+        fns_onsave=None,
+        fns_onrevert=None,
+        **kwargs,
+    ):
+        """creates a widget input form from schema. datatype must be "object"
+
+        Args:
+            schema (dict): json schema defining widget to generate
+            by_alias (bool, optional): use alias in schema. Defaults to False.
+            value (dict, optional): value of json. Defaults to None.
+            update_map_widgets (frozenmap, optional): frozen dict of widgets to map to schema items. Defaults to None.
+            fn_onsave (callable, optional): function to run on save. Defaults to None.
+            fn_onrevert (callable, optional): function to run on revert. Defaults to None.
+
+        Returns:
+            AutoObject(w.VBox)
+        """
+
+        self.update_map_widgets = update_map_widgets
+        self._init_schema(schema, by_alias=by_alias)  # TODO: make schema a trait
+        self._init_ui()
+        super().__init__(fns_onsave=fns_onsave, fns_onrevert=fns_onrevert, **kwargs)
+        self.title.value = make_bold(self.get_title())
+        self.description.value = self.get_description()
+        self._format_rows()
+        if value is not None:
+            self.value = value
+        else:
+            self._value = self.di_widgets_value
 
     @tr.validate("insert_rows")
     def validate_insert_rows(self, proposal):
@@ -411,6 +484,26 @@ class AutoObject(AutoObjectFormLayout):  # w.VBox
                 raise ValueError("keys of insert_rows must be integers")
         return v
 
+    @tr.observe("disabled")
+    def observe_disabled(self, on_change):
+        if self.disabled:
+            for k, v in self.di_widgets.items():
+                try:
+                    v.disabled = True
+                except:
+                    logger.warning(f"{k}: widget does not have a `disabled` traitlet")
+        else:
+            for k, v in self.di_widgets.items():
+                if "disabled" in self.pr[k].schema_ and self.pr[k].schema_["disabled"]:
+                    logger.info(f"{k}: widget disabled in base schema")
+                else:
+                    try:
+                        v.disabled = False
+                    except:
+                        logger.warning(
+                            f"{k}: widget does not have a `disabled` traitlet"
+                        )
+
     @tr.validate("order")
     def _order(self, proposal):
         v = proposal["value"]
@@ -425,7 +518,6 @@ class AutoObject(AutoObjectFormLayout):  # w.VBox
                         "if you want to use order to hide rows then set:"
                         "`order_hides_rows = True`"
                     )
-        # BUG: order not updating
         return v
 
     @tr.validate("order_can_hide_rows")
@@ -480,55 +572,6 @@ class AutoObject(AutoObjectFormLayout):  # w.VBox
             if hasattr(self, "di_widgets"):
                 self._update_widgets_from_value()
 
-    def __init__(
-        self,
-        schema,
-        value=None,
-        by_alias=False,
-        update_map_widgets=None,
-        fdir=None,
-        order=None,
-        insert_rows=None,
-        nested_widgets=None,  # TODO
-        show_description: bool = True,
-        show_title: bool = True,
-        fns_onsave=None,
-        fns_onrevert=None,
-    ):
-        """creates a widget input form from schema. datatype must be "object"
-
-        Args:
-            schema (dict): json schema defining widget to generate
-            value (dict, optional): value of json. Defaults to None.
-            update_map_widgets (frozenmap, optional): frozen dict of widgets to map to schema items. Defaults to None.
-            fdir (path, optional): fdir to work from. useful for widgets that link to files. Defaults to None.
-            order (list): allows user to re-specify the order for widget rows to appear by key name in self.di_widgets
-            insert_rows (dict): e.g. {3:w.Button()}. allows user to insert a widget into the rows. its presence
-                is ignored by the widget otherwise.
-            nested_widgets (list): e.g. [FileUploadToDir]. allows user to indicate widgets that should be show / hide
-                type
-
-        Returns:
-            AutoObject(w.VBox)
-        """
-        super().__init__(fns_onsave=fns_onsave, fns_onrevert=fns_onrevert)
-        self.insert_rows = insert_rows
-        self.update_map_widgets = update_map_widgets
-        self.fdir = fdir
-        self._init_schema(schema, by_alias=by_alias)  # TODO: make schema a trait
-        self._init_ui()
-        self._format_rows()
-        setdefault = lambda val, default: default if val is None else val
-        self.order = setdefault(order, None)
-        self.show_description = show_description
-        self.show_title = show_title
-        self.title.value = make_bold(self.get_title())
-        self.description.value = self.get_description()
-        if value is not None:
-            self.value = value
-        else:
-            self._value = self.di_widgets_value
-
     def display_showraw(self):
         self.autowidget.layout.display = "None"
         return self.json
@@ -540,10 +583,10 @@ class AutoObject(AutoObjectFormLayout):  # w.VBox
         else:
             return json.dumps(self.value, indent=4)
 
-    def get_description(self):  # TODO: put this in AutoObjectFormLayout
+    def get_description(self):
         return get_from_schema_root(self.schema, "description")
 
-    def get_title(self):  # TODO: put this in AutoObjectFormLayout
+    def get_title(self):
         return get_from_schema_root(self.schema, "title")
 
     def _init_ui(self):
@@ -552,10 +595,6 @@ class AutoObject(AutoObjectFormLayout):  # w.VBox
 
     def _init_schema(self, schema, by_alias=False):
         self.model, self.schema = aumap._init_model_schema(schema, by_alias=by_alias)
-        # if "type" not in self.schema.keys() and self.schema["type"] != "object":
-        #     raise ValueError(
-        #         '"type" must be in schema keys and "type" must == "object"'
-        #     )
         if "properties" in self.schema.keys():
             pr = self.schema["properties"]
         else:
@@ -566,20 +605,18 @@ class AutoObject(AutoObjectFormLayout):  # w.VBox
             )
             for property_key, property_schema in pr.items()
         }
+        self.contains_nullable = False
+        for v in self.pr.values():
+            if "nullable" in v.schema_:
+                self.contains_nullable = True
+                break
+
         if self.fdir is not None:
             for v in self.pr.values():
                 v = add_fdir_to_widgetcaller(v, self.fdir)
 
-    # def is_nullable(self, key):
-    #     if "nullable" in self.pr[key].keys() and self.pr[key]["nullable"]:
-    #         return True
-    #     else:
-    #         return False
-
     def _init_widgets(self):
         self.di_labels, self.di_widgets = _init_widgets_and_labels(self.pr)
-        # self._value = self.di_widgets_value
-        # self._update_widgets_from_value()
 
     def _insert_rows(self):
         if self.insert_rows is not None:
@@ -596,6 +633,7 @@ class AutoObject(AutoObjectFormLayout):  # w.VBox
                 align_horizontal=self.align_horizontal,
                 auto_open=self.auto_open,
                 nested_widgets=self.nested_widgets,
+                contains_nullable=self.contains_nullable,
             )
             for row in order
         ]
@@ -636,7 +674,14 @@ class AutoObject(AutoObjectFormLayout):  # w.VBox
         )
 
     def _watch_change(self, change, key=None, watch="value"):
-        # print(f"changed: {key}")
+        # if self.validate_on_change:
+        #     if self.model is not None:
+        #         new_value = json.loads(self.model(**self.di_widgets_value).json())
+        #     else:
+        #         raise ValueError(
+        #             "currently a pydantic model is required to validate on change"
+        #         )
+
         self._value = self.di_widgets_value
         self.savebuttonbar.unsaved_changes = True
         # NOTE: it is required to set the whole "_value" otherwise
@@ -678,16 +723,28 @@ class AutoObject(AutoObjectFormLayout):  # w.VBox
 if __name__ == "__main__":
     from ipyautoui.demo_schemas import CoreIpywidgets
 
-    ui = AutoObject(CoreIpywidgets)
+    ui = AutoObject(
+        CoreIpywidgets,
+        order=["text", "int_text", "int_slider", "int_slider_nullable"],
+        show_description=False,
+        show_title=False,
+        show_savebuttonbar=False,
+        show_raw=False,
+    )
     display(ui)
 
+# +
+# ui.show_savebuttonbar = True
 
+# +
 if __name__ == "__main__":
-    ui.order = [
+    order = [
         "text",
         "int_text",
         "int_slider",
-        # "int_range_slider",
+        "int_slider_nullable",
+        "int_range_slider",
+        "int_range_slider_disabled",
         # "float_slider",
         # "float_text",
         # "float_text_locked",
@@ -696,15 +753,27 @@ if __name__ == "__main__":
         # "dropdown",
         # "dropdown_edge_case",
         # "dropdown_simple",
-        "combobox",
         "text_short",
         "text_area",
-        "auto_markdown",
     ]
+    ui.order = order
+
+# "combobox", "auto_markdown"
+# -
+
+if __name__ == "__main__":
+    ui.disabled = True
+
+if __name__ == "__main__":
+    ui.disabled = False
 
 if __name__ == "__main__":
     ui.align_horizontal = False
 
+if __name__ == "__main__":
+    ui.align_horizontal = True
+
+# +
 if __name__ == "__main__":
     ui.nested_widgets = []
 
