@@ -124,6 +124,14 @@ def add_nullable_to_object(schema_obj):
         req = schema_obj["required"]
     else:
         req = []
+    if "properties" not in schema_obj.keys():
+        if "autoui" in schema_obj.keys():
+            return schema_obj
+        else:
+            raise ValueError(
+                "AutoUi does not support rendering generic dictionaries."
+                " This can be overridden by specifying a `autoui` pyobject renderer."
+                )
     for k, v in schema_obj["properties"].items():
         if k not in req and "default" not in v.keys():
             v["nullable"] = True
@@ -617,7 +625,6 @@ def update_widgets_map(widgets_map, di_update=None):
 
 
 def get_widgets_map(di_update=None):
-
     WIDGETS_MAP = frozenmap(
         **{
             "AutoOveride": WidgetMapper(
