@@ -1,16 +1,13 @@
-import sys
-import pathlib
+# import sys
+# import pathlib
 
-sys.path.append(str(pathlib.Path(__file__).parents[2]))
+# sys.path.append(str(pathlib.Path(__file__).parents[2]))
 # ^ for dev only. TODO: comment out at build time
 
 from ipyautoui.custom.filechooser import FileChooser
 from ipyautoui.custom.multiselect_search import MultiSelectSearch
 from ipyautoui.custom.iterable import Array, Dictionary
-
-# from ipyautoui.custom.modelrun import RunName
 from ipyautoui.custom.loadproject import LoadProject
-from ipyautoui.custom.save_buttonbar import SaveButtonBar
+from ipyautoui.custom.buttonbars import SaveButtonBar
 
 # from ipyautoui.custom.edit_menu import EditGrid  # TODO: make this work
-
